package de.fitconnect.client;

import fitconnect.api.Sender;
import fitconnect.api.auth.OAuthToken;
import fitconnect.api.domain.metadata.Metadata;
import fitconnect.api.domain.metadata.attachment.Attachment;
import fitconnect.api.domain.metadata.data.Data;
import fitconnect.api.domain.submission.Submission;
import fitconnect.api.logger.SdkLogger;

import java.util.List;
import java.util.Optional;
import java.util.logging.Level;
import java.util.logging.Logger;

public class FitConnectClient {

    private static final Logger logger = SdkLogger.defaultLogger(FitConnectClient.class);

    private final Sender sender;
    private Optional<OAuthToken> token = Optional.empty();

    public FitConnectClient(){
        this.sender = ClientFactory.submissionSender();
    }
    
    public FitConnectClient authenticate(String clientId, String secret, String...scope){
        this.token = sender.retrieveOAuthToken(clientId,secret,scope);
        return this;
    }

    public FitConnectClient createSubmission(final Submission submission){
        checkIfAuthenticated();
        // TODO implement sendSubmission
        logger.log(Level.INFO, "created new submission");
        return this;
    }

    public FitConnectClient sendSubmission(final Metadata metadata, final Data data){
        checkIfAuthenticated();
        // TODO implement
        logger.log(Level.INFO, "successfully sent submission");

        return this;
    }

    public FitConnectClient uploadAttachments(final List<Attachment> attachments){
        checkIfAuthenticated();
        // TODO implement
        logger.log(Level.INFO, "uploaded " + attachments.size() + " attachments");
        return this;
    }

    public FitConnectClient printToken(){
        checkIfAuthenticated();
        logger.log(Level.INFO, "retrieved access token: " + this.token.get().access_token());
        return this;
    }

    private void checkIfAuthenticated() {
        if(token.isEmpty()){
            throw new IllegalStateException("not authenticated, please authenticate first with authenticate(client, secret)");
        }
    }

}
